@@ -15,7 +15,7 @@
 #define E_BOOK_BACKEND_CHECK_METHOD(backend, method, ...) G_STMT_START{ \
 	if (G_UNLIKELY (!E_BOOK_BACKEND_GET_CLASS ((backend))->method)) { \
 		g_warning ("%s: %s: unsupported method: %s", G_STRFUNC, G_OBJECT_TYPE_NAME ((backend)), #method); \
-		e_data_book_respond_ ## method (book, opid, OtherError, ## __VA_ARGS__); \
+		e_data_book_respond_ ## method (book, opid, MethodNotSupported, ## __VA_ARGS__); \
 		return; \
 	} \
 }G_STMT_END
@@ -505,6 +505,35 @@ e_book_backend_get_changes (EBookBackend *backend,
 	(* E_BOOK_BACKEND_GET_CLASS (backend)->get_changes) (backend, book, opid, change_id);
 }
 
+/**
+ * e_book_backend_reset_changes:
+ * @backend: an #EBookBackend
+ * @book: an #EDataBook
+ * @opid: the ID to use for this operation
+ * @change_id: the ID of the changeset
+ *
+ * Executes a 'reset changes' request specified by @opid on @book
+ * using @backend.
+ */
+void
+e_book_backend_reset_changes (EBookBackend *backend,
+			      EDataBook    *book,
+			      guint32       opid,
+			      const char   *change_id)
+{
+	g_return_if_fail (E_IS_BOOK_BACKEND (backend));
+	g_return_if_fail (E_IS_DATA_BOOK (book));
+	g_return_if_fail (change_id);
+
+	E_BOOK_BACKEND_CHECK_METHOD (backend, reset_changes);
+
+	e_flag_wait (backend->priv->opened_flag);
+
+	E_BOOK_BACKEND_CHECK_FLAG(backend, loaded, reset_changes, OtherError);
+
+	(* E_BOOK_BACKEND_GET_CLASS (backend)->reset_changes) (backend, book, opid, change_id);
+}
+
 /**
  * e_book_backend_authenticate_user:
  * @backend: an #EBookBackend
@@ -644,14 +673,23 @@ last_client_gone (EBookBackend *backend)
  *
  * Gets the list of #EDataBookView views running on this backend.
  *
- * Return value: An #EList of #EDataBookView objects.
+ * Return value: An #EList of #EDataBookView objects,
+ * caller should unref the result with #g_object_unref().
  **/
 EList*
 e_book_backend_get_book_views (EBookBackend *backend)
 {
+	EList *copy;
+
 	g_return_val_if_fail (E_IS_BOOK_BACKEND (backend), NULL);
 
-	return g_object_ref (backend->priv->views);
+	g_mutex_lock (backend->priv->views_mutex);
+
+	copy = e_list_duplicate (backend->priv->views);
+
+	g_mutex_unlock (backend->priv->views_mutex);
+
+	return copy;
 }
 
 /**
@@ -974,12 +1012,12 @@ e_book_backend_foreach_view (EBookBackend *backend,
 			     void (*callback) (EDataBookView *, gpointer),
 			     gpointer user_data)
 {
-	EList *views;
 	EDataBookView *view;
 	EIterator *iter;
 
-	views = e_book_backend_get_book_views (backend);
-	iter = e_list_get_iterator (views);
+	g_mutex_lock (backend->priv->views_mutex);
+
+	iter = e_list_get_iterator (backend->priv->views);
 
 	while (e_iterator_is_valid (iter)) {
 		view = (EDataBookView*)e_iterator_get (iter);
@@ -991,8 +1029,10 @@ e_book_backend_foreach_view (EBookBackend *backend,
 		e_iterator_next (iter);
 	}
 
+	e_list_remove_iterator (backend->priv->views, iter);
 	g_object_unref (iter);
-	g_object_unref (views);
+
+	g_mutex_unlock (backend->priv->views_mutex);
 }
 
 
